function bcomplement(q,n,m)
% bcomplement(q,n,m)
% Interaktive Darstellung der Reprsentation ganzer Zahlen 
% im Stellensystem zur Basis  1 < q < 65 mit n Stellen.
% Die Basis q, die Anzahl der Stellen und die maximale positive
% Zahl m knnen interaktiv gendert werden.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  q = 2; end
if nargin<2  n = 4; end
if nargin<3  m = floor((q^n - 1)/2); end

% Konstanten
% ==========
scrs = get(0,'ScreenSize');
fsiz = round(20/1024*scrs(3));                   % FontSize
pcol = 0.6*[1 1 1];                              % Untergrundfarbe fr Rechtecke
pcol2= 0.9*[1 1 1];                              % Farbe fr Rechtecke
d    = 0.05;

txt  = {'Reprsentation';                        % Texte
        'Information';
        'Basis';
        'Stellen';
        'grsste Zahl';
        'Interpretation'};
     
helptxt = {'Hilfe zu BCOMPLEMENT:';               % Hilfetext
           'Nach Klicken in die Editierfenster fr';
           'Basis / Stellen / grsste Zahl knnen'
           'diese Werte editiert und mit der';
           'Eingabetaste abgeschlossen werden.'};
     
fsz  = [(fsiz+5)*[1 1]  fsiz*[1 1 1]  fsiz-3  fsiz-2  fsiz-2];   % FontSize
     
col  = [zeros(2,3); ones(3,3);                   % Farben 
        1 0 0
        0 0 0];
     
ppos = [0  0.54];                                % x-Position der grauen Patches
dx   = 0.038;
dy   = 0.0596;
yu   = -1+0.02;
yo   = yu + (q^n)*dy;
ppos2= [ppos(1)-0.05-n*dx  ppos(1)-0.02  yu  yo;
        ppos(2)+0.02       ppos(2)+0.2   yu  yo];

upos = [-1.35   0.4;                             % Position der UIs
        -1.35  -0.1;
        -1.35  -0.6];
      
tpos = [-1.35           0.8;                     % Position der Texte
         ppos(2)+0.3    0.8;
         upos(1,1)+0.2  upos(1,2)+0.05;
         upos(2,1)+0.2  upos(2,2)+0.05;
         upos(3,1)+0.2  upos(3,2)+0.05;
         ppos(1)+0.05   0.05;
        -ppos(1)-0.01  -0.96;
         ppos(2)+0.15  -0.96];
     
scs  = get(0,'ScreenSize');
if scs(3)>(1.5*scs(4))                           % Skalenfaktoren whlen  
   skv = 0.5;
   skh = skv/1.5;
else 
   skh = 0.5/1.5;
   skv = skh*1.5;
end

% Bild und Achsen
% ===============
fh = fullscreen('B-Compl');                      % Schirmbild
scs = get(0,'ScreenSize');                       % Bildschirmgrsse in Pixel
if scs(3)>(1.5*scs(4))                           % Achsenposition whlen
   a = 1 - 4/3*scs(4)/scs(3)
   b = 0;
else 
   a = 0;
   b = 1 - 3/4*scs(3)/scs(4);
end
ax = axes('Position',[a/2 b/2 1-a 1-b]);         % Ganzbild-Achsen
axis([-1.5 +1.5 -1 +1]); axis off                % Einheiten

% Graue Flchen fr Information und Reprsentation
% ================================================
ph(1) = patch([-1.5  ppos(1) ppos(1) -1.5], [-1 -1 1 1],pcol);
ph(2) = patch([+1.5  ppos(2) ppos(2) +1.5], [-1 -1 1 1],pcol);

% Helle Flchen fr Information und Reprsentation
% ================================================
ph(3) = patch([ppos2(1,1) ppos2(1,2) ppos2(1,2) ppos2(1,1)], [ppos2(1,3) ppos2(1,3) ppos2(1,4) ppos2(1,4)],pcol2);
ph(4) = patch([ppos2(2,1) ppos2(2,2) ppos2(2,2) ppos2(2,1)], [ppos2(2,3) ppos2(2,3) ppos2(2,4) ppos2(2,4)],pcol2);

% Pfeil
% =====
drawarr([ppos(1) 0], [ppos(2) 0],'r',2,0.05)

% Alle Texte
% ==========
for k=1:length(txt)
   htx(k) = text(tpos(k,1),tpos(k,2),txt{k},'FontSize',fsz(k),'Color',col(k,:));
end

% Textfeld Reprsentation
% =======================
for k=0:32
   thr(k+1) = text(tpos(7,1),tpos(7,2)+0.06*k,num2basstr(k,q,12),'FontSize',fsz(7),'Color',col(7,:),...
                   'HorizontalAlignment','right','FontName','FixedWidth','FontWeight','demi');
end

% Textfeld Interpretatation
% =========================
for k=0:32
   str = num2basstr(k,q,n);
   thi(k+1) = text(tpos(8,1),tpos(8,2)+0.06*k,num2str(bcomplstr2num(str(end-n+1:end),q,m)),'FontSize',fsz(7),'Color',col(7,:),...
                   'HorizontalAlignment','right','FontName','FixedWidth','FontWeight','demi');
end

% Interaktivitt
% ==============
uip = axpos2figpos(upos);
uiq = uicontrol(fh,'Units','normalized','Position',[uip(1,:) 0.05 0.05],'String',num2str(q),...
                   'Style','edit','FontSize',fsiz,'FontName','FixedWidth',...
                   'CallBack','bcomplementfcn(1)','BackgroundColor',[1 1 1]);
                              
uin = uicontrol(fh,'Units','normalized','Position',[uip(2,:) 0.05 0.05],'String',num2str(n),...
                   'Style','edit','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','bcomplementfcn(2)','BackgroundColor',[1 1 1]);
                              
uim = uicontrol(fh,'Units','normalized','Position',[uip(3,:) 0.05 0.05],'String',num2str(m),...
                   'Style','edit','FontSize',fsiz,'FontName','FixedWidth',...
                   'CallBack','bcomplementfcn(3)','BackgroundColor',[1 1 1]);
                              
% UserData des Fensters fr die Interaktivitt
% ============================================
fud.helptext   = helptxt;
fud.uihandles  = [uiq  uin  uim];
fud.rephandles = thr;
fud.infhandles = thi;
fud.pathandles = ph(3:4);
fud.q          = q;
fud.n          = n;
fud.m          = m;
set(fh,'UserData',fud,'KeyPressFcn','helpfcn');
