function bintab = binexp2bintab(binexp,variable,tab)
% bintab = binexp2bintab(binexp,variable)
% Berechnung der Funktionstabelle bintab fr einen als String binexp
% gegebenen schaltalgebraischen Ausdruck mit den Variablen variable.
% Der Ausdruck darf in MATLAB-Syntax alle binren Operatoren enthalten
% und zustzlich die GF(2)-Verknpfungen * und +
%
% binexp   :  String fr den schaltalgebraischen Ausdruck
%             Beispiel:  '(a1|b) & (~a2|c) & b|c'
%
% variable :  Cell array mit den Variablen als Strings
%             Folgende Variablennamen sind nicht erlaubt:
%             iiiii,jjjjj,kkkkk,nnnnn,binexp,bintab,eval,sprintf,variable
%             sowie alle Schlsselwrter von MATLAB wie for, end, ...
%             Beispiel:  {'a1'; 'a2'; 'b'; 'c'}
%
% tab      :  wenn tab mit (beliebigem Wert) angegeben, dann
%             Ausgabe der Funktionstabelle bintab als String
%             andernfalls als Binrvektor.
% tab = 2  :  Die Funktionstabelle wird in einem Fenster ausgegeben.
%
% binexp darf Anweisungen enthalten. Der letzte Ausdruck liefert den 
% Funktionswert.  Beispiel:
% binexp2bintab('a2=1; p=0; (a1|b) & (~a2|c) & p|c', {'a1'; 'b'; 'c'})
%
% Siehe auch binexp2bintab3 (ohne weitere Anweisungen, mit *,+ etc.)

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<2  variable = {'a';'b';'c';'d'}; end
if nargin<1  binexp = '(a&c) | (~b&d&~c)'; end

% Konstante
% =========
nnnnn = length(variable);           % Anzahl der Variablen

% Berechnung der Funktionstabelle
% ===============================
if any(strcmp(binexp,variable))  binexp = ['1&' binexp]; end
%binexp = ['1&' binexp];
%binexp = ['mod(' binexp ',2)']
for jjjjj=0:(2^nnnnn-1)             % Schleife ber alle ...
   kkkkk = num2binvec(jjjjj,nnnnn); % Input-Bitmuster             
   for iiiii=1:nnnnn                % alle Variablen setzen
      eval(sprintf('%s=kkkkk(%d);',variable{iiiii},iiiii));
   end
   try
      eval([binexp ';'])            % Ausdruck evaluieren
      bintab(jjjjj+1) = ans;        % Resultat in Tabelle
   catch
      bintab(jjjjj+1) = NaN;        % Ausdruck liefert NaN
   end
end

if nargin<3  break; end

% Ausgabe als Stringarray
% =======================
l = 20;                             % Zeilenlnge bei Textausgabe
b = '01';                           % Bits als Strings
for v=1:nnnnn
   z = '';
   for w=1:v-1  z = [z '| ']; end
   vv = variable(v);
   z = [z  cat(1,vv{:})];
   for w=length(z)+1:l  z = [z  ' ']; end
   t(v,:) = z;
end
for k=0:(2^nnnnn-1)
   bin = binmatout(num2binvec(k,nnnnn));
   z = reshape([bin(1:end-1); blanks(nnnnn)],1,2*nnnnn);
   z = [z  '|  '  b(bintab(k+1)+1)];
   for w=length(z)+1:l  z = [z  ' ']; end
   t(k+nnnnn+1,:) = z;
end

bintab = t;

if tab==2
   if nnnnn<5  fsiz = 21; else fsiz = 13; end
   fh = fullscreen('Funktionstabelle','w',0,[1 4; 1 1]);
   axes('Position',[0 0 1 1]); 
   text(1,0,t,'FontName','FixedWidth','FontSize',fsiz); 
   axis([0 2*nnnnn+5  [-1 +1]*2^(nnnnn-1)]); axis off
end