function bintab = binexp2bintab2(binexp,variable,tab)
% bintab = binexp2bintab2(binexp,variable,tab)
% Berechnung der Funktionstabelle bintab fr einen als String binexp
% gegebenen schaltalgebraischen Ausdruck mit den Variablen variable. 
%
% binexp   :  String fr den schaltalgebraischen Ausdruck
%             Beispiel:  '(a1|b) & (~a2|c) & b|c'
%
% variable :  Cell array mit den Variablen als Strings
%             Die Variablennamen mssen den Einschrnkungen von symvar gengen
%             Erstes Zeichen ist ein Buchstabe, danach beliebige Buchstaben, 
%             Ziffern oder Unterstrich.  Vordefinierte Konstanten wie 
%             'i', 'j', 'pi', 'inf', 'nan', 'eps', 'sin', etc. sind nicht erlaubt.
%             Auch die Schlsselwrter von MATLAB wie for, while, end, case etc. 
%             sind nicht erlaubt.  Defaults sind:  a, b, c, d.
%             Beispiel:  {'a1'; 'a2'; 'b'; 'c'}
%
% tab      :  wenn tab mit (beliebigem Wert) angegeben, dann
%             Ausgabe der Funktionstabelle bintab als String
%             andernfalls als Binrvektor
%
% !!! binexp2bintab tut das selbe mit etwas geringeren Einschrnkungen fr 
% !!! die Variablen bei der zustzlichen Mglichkeit, weitere Instruktionen 
% !!! in binexp ausfhren zu lassen.
%
% Beispiel:  binexp2bintab2('(a1|a2) & (~a2|c) & a1|c', {'a1'; 'a2'; 'c'},1)

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<2  variable = {'a';'b';'c';'d'};   end
if nargin<1    binexp = '(a&c) | (~b&d&~c)'; end

% Konstanten
% ==========
n = length(variable);                       % Anzahl der Variablen

% Temporre Variablennamen
% ========================
nvar = {};
for k=1:n
   nvar{k} = sprintf('a%d',k);
end
tbinexp = replacevariablenames(binexp,variable,nvar);

% Berechnung der Funktionstabelle
% ===============================
for j=0:(2^n-1)                             % Schleife ber alle ...
   x = num2binvec(j,n);                     % Input-Bitmuster             
   for i=1:n  
      eval(sprintf('%s=x(%d);',nvar{i},i)); % Variable setzen
   end
   eval(['bintab(j+1)=',tbinexp,';']);      % Ausdruck evaluieren
end

if nargin<3  break; end

% Ausgabe als Stringarray
% =======================
l = 20;                                     % Zeilenlnge bei Textausgabe
b = '01';                                   % Bits als Strings
for v=1:n
   z = '';
   for w=1:v-1  z = [z '| ']; end
   vv = variable(v);
   z = [z  cat(1,vv{:})];
   for w=length(z)+1:l  z = [z  ' ']; end
   t(v,:) = z;
end
for k=0:(2^n-1)
   bin = binmatout(num2binvec(k,n));
   z = reshape([bin(1:end-1); blanks(n)],1,2*n);
   z = [z  '|  '  b(bintab(k+1)+1)];
   for w=length(z)+1:l  z = [z  ' ']; end
   t(k+n+1,:) = z;
end

bintab = t;