function ft = binexp2flusstafel(binexp,invar,tab)
% ft = binexp2flusstafel(binexp,invar,tab)
% Berechnung der Flusstafel ft fr die als schaltalgebraische Ausdrcke
% in den Variablen variable gegebenen Zustandsvariablen.
% Der Ausdruck darf in MATLAB-Syntax alle binren Operatoren enthalten
% und zustzlich die GF(2)-Verknpfungen * und +
%
% binexp   :  CellArray von Strings der Ausdrcke fr die Folgezustnde
%             in der Reihenfolge z1, z2,...
%             Die Zustandvariablen haben die festen Bezeichner z1, z2, ... , z9
%
% invar    :  Cell array mit den Belegungen der Inputvariablen in den 
%             Spalten der Flusstafel als Strings.
%             z.B.: {'D=0; C=0;'; 'D=0; C=1;'; 'D=1; C=1;'; 'D=1; C=0;'}
%             Folgende Variablennamen sind nicht erlaubt:
%             iiiii,jjjjj,kkkkk,lllll,nnnnn,binexp,binexpp,bintab,...
%             z1, z2, ... , z9 (weil so schon die Zustandsbits heissen),
%             eval,sprintf,variable sowie alle Schlsselwrter von MATLAB 
%             wie for, end, ...
%
% tab      :  wenn tab mit (beliebigem Wert) angegeben, dann
%             Ausgabe der Flusstafel als String
%             andernfalls als Binrvektor.
% tab = 2  :  Die Flusstafel wird in einem Fenster ausgegeben.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Konstanten
% ==========
nnnnn = length(invar);                    % Anzahl der Spalten der Flusstafel
mmmmm = length(binexp);

% Berechnung der Spalten der Flusstafel
% =====================================
for kkkkk=1:nnnnn                         % Spaltenindex (Inputkombination)
   for lllll=1:mmmmm                      % Subspalte (Zustandsbit)
      for jjjjj=0:(2^mmmmm-1)             % Schleife ber alle ...
         ooooo = num2binvec(jjjjj,mmmmm); % Input-Bitmuster             
         for iiiii=1:mmmmm                % alle Variablen setzen
            eval(['z' num2str(iiiii) ' = ' num2str(ooooo(iiiii)) ';']);
         end
         eval([invar{kkkkk} 'ftt(jjjjj+1,lllll) = mod(' binexp{lllll} ',2);'])  % Ausdruck evaluieren
      end
   end
   ft(:,kkkkk) = 1 + binmat2num(ftt)';
end

if nargin<3  break; end

% Ausgabe als Stringarray
% =======================
%if iscell(binexp)  nf = length(binexp); else nf = 1; end
if iscell(binexp)  nf = length(invar); else nf = 1; end
l  = 18 + 2*nf;                           % Zeilenlnge bei Textausgabe
b  = '01';                                % Bits als Strings
for v=1:nnnnn
   z = '';
   for w=1:v-1  z = [z '| ']; end
   vv = invar(v);
   z = [z  cat(1,vv{:})];
   for w=length(z)+1:l  z = [z  ' ']; end
   t(v,:) = z;
end
for k=0:(2^nnnnn-1)
   bin = binmatout(num2binvec(k,nnnnn));
   z = reshape([bin(1:end-1); blanks(nnnnn)],1,2*nnnnn);
   if nf==1
      z = [z  '|  '  b(ft(k+1)+1)];
   else
      z = [z  '|  '];
      for j=1:nf
         z = [z  ' '  b(ft(k+1,j)+1)];
      end
   end
   for w=length(z)+1:l  z = [z  ' ']; end
   t(k+nnnnn+1,:) = z;
end

ft = t;

if tab==2
   if nnnnn<5  fsiz = 21; else fsiz = 13; end
   fh = fullscreen('Funktionstabelle','w',0,[0 0.2+nf*0.04 0 1]);
   axes('Position',[0 0 1 1]); 
   text(1,0,t,'FontName','FixedWidth','FontSize',fsiz); 
   axis([0 2*nnnnn+5  [-1 +1]*2^(nnnnn-1)]); axis off
end