function  [mt,at] = demomorserobust(n)
% [mt,at] = demomorserobust(n)
% Der Text 'DIES IST DER KORREKTE TEXT.' wird in Morsezeichen und diese in Bits
% umgesetzt:  Punkt nach 1, Strich nach 01, Zeichentrennung nach 00.
% Dem Bitvektor wird eine zufllige Anzahl von 0...n-1 zuflliger Bits 
% vorangestellt. Dann wird dieser Bitvektor dekodiert um zu zeigen, dass 
% der Code auch nach beliebigen Fehlern sofort wieder dekodiert werden kann.
% Zum Vergleich wird das selbe mit der ASCII-Codierung durchgefhrt. 
% In diesem Fall gelingt das Lesen nur, wenn die Anzahl der eingefgten 
% Bits ein Vielfaches von 8 ist.
% Wenn die Ausgabetexte nicht zugewiesen werden, gibt es eine graphische Ausgabe.

% Autor:  K. v. d. Heide,  e-mail:  v.d.heide@on-line.de

if nargin<1  n = 32; end

fsiz = 20;

str = 'DIES IST DER KORREKTE TEXT.';                   % zu bertragender String
vb  = randb(floor(n*rand));                            % zufllige Bits als Vorlauf

% Umsetzung in Morsecode, Voranstellen zuflliger Bits und Decodierung
% ====================================================================
mt  = morsebits2text([vb  morse2bits(str2morse(str))]);% Umsetzung in Morse und zurck

% Umsetzung in ASCII, Voranstellen zuflliger Bits und Decodierung
% ================================================================
bm  = num2binmat(double(str)',8);                      % Binrmatrix
[mbm,nbm] = size(bm);                                  % Grsse der Binrmatrix
bv  = reshape(bm',1,mbm*nbm);                          % Binrvektor
bvv = [vb  bv];                                        % Voranstellen zuflliger Bits
bvv = [bvv zeros(1,8-mod(length(bvv),8))];             % Verlngern auf Vielfaches von 8
at  = char(binmat2num(reshape(bvv,8,length(bvv)/8)')); % Dekodierung des ASCII-Codes

if nargout>0  break; end

% Graphische Ausgabe
% ==================
fh = fullscreen('MorseDemo','w',0,[3 1; 1 1]);
axes('Position',[0 0 1 1])
axis off

text(0.1,0.72,'Morse-Code:','FontSize',fsiz,'FontName','Times','Color','b','FontAngle','italic')
text(0.3,0.7,mt,'FontSize',fsiz,'FontName','FixedWidth','Color','k','FontWeight','bold','Interpreter','none')
ix = findstr(str(2:end),mt);
if str(1)==mt(ix-1)  ix = ix - 1; end
text(0.3,0.7,[mt(1:ix-1) ' '],'FontSize',fsiz,'FontName','FixedWidth','Color','r','FontWeight','bold','Interpreter','none')

text(0.1,0.42,'ASCII-Code:','FontSize',fsiz,'FontName','Times','Color','b','FontAngle','italic')
text(0.3,0.4,at,'FontSize',fsiz,'FontName','FixedWidth','Color','k','FontWeight','bold','Interpreter','none')
ix = findstr(str(2:end),at);
if isempty(ix) 
   ix = length(at) + 1;
elseif str(1)==at(ix-1)  ix = ix - 1; 
end
text(0.3,0.4,[at(1:ix-1) ' '],'FontSize',fsiz,'FontName','FixedWidth','Color','r','FontWeight','bold','Interpreter','none')
