function [ta,tm] = gftabellen(q)
% [ta,tm] = gftabellen(q)
% Erzeugung einer Additions- und Multiplikationstabelle ta bzw. tm 
% fr den endlichen Krper  GF(q).  q muss Primzahlpotenz sein.
%
% Als Elemente des GF(q) werden die natrlichen Zahlen 1 ... q verwendet.
% Das Eins-Element der Addition (die sog. Null) ist 1,
% das Eins-Element der Multiplikation (die sog. Eins) ist 2. 
% Damit knnen die Tabellen direkt geschachtelt verwendet werden.
%
% Beispiel:       zu berechnen in GF(8) ist  3*(1+8*2)
% Realisierung:   gf.mul(3,gf.add(1,gf.mul(8,2)))
%
% Hinweis:  wegen Instabilitten in den benutzten Funktionen  gf_new
%           und gf_mul ist die Berechnung fr q = 5^k und 7^k (k>1) 
%           vorerst nicht mglich!

% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de

if q==2
   ta = 1 + [0 1; 1 0];
   tm = 1 + [0 0; 0 1]; 
else
   fq = factor(q);
   if length(fq)==1
      for a=1:q
         for b=1:q
            ta(a,b) = 1 + rem(a+b-2,q);
            tm(a,b) = 1 + rem((a-1)*(b-1),q);
         end
      end
   else
      if q~=fq(1)^length(fq) error('gftab: q muss Primzahlpotenz sein'); end
      
      if fq(1)==5 | fq(1)==7 
         error('gftabellen: q=5^k und q=7^k mit k>1 gehen zur Zeit noch nicht!')
      end
      
      p  = fq(1);
      n  = length(fq);

      for a=1:q   ap(a,:) = num2basvec(a-1,p,n); end

      w = 1;
      while w
         try
            gf = gf_new(p,n);
            for a=1:q 
               for b=a:q
                  gfaddtab(a,b) = basvec2num(rem(ap(a,:)+ap(b,:),p),p);
                  gfmultab(a,b) = basvec2num(gf_mul(gf,ap(a,:),ap(b,:)),p); 
                  gfaddtab(b,a) = gfaddtab(a,b);   
                  gfmultab(b,a) = gfmultab(a,b);   
               end 
            end
            w = 0;
         end
      end
      ta = 1 + gfaddtab;
      tm = 1 + gfmultab;
   end
end