function inforepres
% inforepres
% Interaktive Darstellung der Reprsentation natrlicher Zahlen 
% im Stellensystem zur Basis  1 < b < 65.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Konstanten
% ==========
scrs = get(0,'ScreenSize');
fsiz = round(20/1024*scrs(3));                   % FontSize
pcol = 0.6*[1 1 1];                              % Untergrundfarbe fr Rechtecke
d    = 0.05;

txt  = {'Reprsentation';                        % Texte
        'Information';
        'Basis';
        'Stellen';
        'a';
        'b';
        'Operation';
        'Resultate';
        '\alpha';
        '\sigma';
        '\nu';
        'Interpretation'};
     
helptxt = {'Hilfe zu INFOREPRES:';               % Hilfetext
           'nach Klicken in die Fenster fr';
           'Basis / Stellen / a / b / Operation';
           'knnen diese Werte editiert und mit';
           'der Eingabetaste abgeschlossen werden.';
           'Als Operation sind + - * / rem / max / min';
           'mglich.'};
     
fsz  = [(fsiz+5)*[1 1] fsiz*ones(1,6) (fsiz+5)*[1 1 1] fsiz-3]; % FontSize
     
col  = [zeros(2,3);                              % Farben 
        ones(4,3);
        0 1 0;
        1 1 1;
        1 0 0;
        0 1 0;
        0 1 0;
        1 0 0];
     
ppos = [0.2 1.2 -0.8 0.65];                      % Position der grauen Patches

upos = [-ppos(2)+0.11  ppos(4)-0.25;             % Position der UIs
        -ppos(1)-0.26  ppos(4)-0.25;
         ppos(1)+0.1   ppos(3)-d+1.0; 
         ppos(1)+0.1   ppos(3)-d+0.8;
         ppos(1)+0.385 ppos(3)-d+0.55;
         ppos(1)+0.1   ppos(3)-d+0.2;
        -ppos(2)+0.11  ppos(3)-d+1.0;
        -ppos(2)+0.11  ppos(3)-d+0.8;
        -ppos(2)+0.11  ppos(3)-d+0.2;
         ppos(1)+0.1   ppos(3)-d+0.3];
      
tpos = [-ppos(2)       ppos(4)+0.12;             % Position der Texte
         ppos(1)       ppos(4)+0.12;
        -ppos(2)+0.09  ppos(4)-0.1;
        -ppos(1)-0.3   ppos(4)-0.1;
         ppos(2)-0.4   ppos(3)+1.0;
         ppos(2)-0.4   ppos(3)+0.8;
         ppos(2)-0.4   ppos(3)+0.55;
         ppos(2)-0.4   ppos(3)+0.25;
        -0.02          0.1;
         ppos(1)+0.2   ppos(3)+0.55;
        -ppos(1)-0.6   ppos(3)+0.55;
        -0.195         ppos(3)+0.55];
     
scs  = get(0,'ScreenSize');
if scs(3)>(1.5*scs(4))                           % Skalenfaktoren whlen  
   skv = 0.5;
   skh = skv/1.5;
else 
   skh = 0.5/1.5;
   skv = skh*1.5;
end

% Bild und Achsen
% ===============
fh = fullscreen('infrepr');                      % Schirmbild
scs = get(0,'ScreenSize');                       % Bildschirmgrsse in Pixel
if scs(3)>(1.5*scs(4))                           % Achsenposition whlen
   a = 1 - 4/3*scs(4)/scs(3)
   b = 0;
else 
   a = 0;
   b = 1 - 3/4*scs(3)/scs(4);
end
ax = axes('Position',[a/2 b/2 1-a 1-b]);         % Ganzbild-Achsen
axis([-1.5 +1.5 -1 +1]); axis off                % Einheiten

% Graue Flchen fr Information und Reprsentation
% ================================================
ss = -1;
for k=1:2
   ss = -ss;
   ph(k) = patch(ss*ppos([1 1 2 2]),ppos([3 4 4 3]),pcol);
end

% Pfeile
% ======
drawarr([upos(7,1)+0.15 upos(7,2)+0.05],[upos(3,1) upos(3,2)+0.05],'r',2,0.05)
drawarr([upos(8,1)+0.15 upos(8,2)+0.05],[upos(4,1) upos(4,2)+0.05],'r',2,0.05)
drawarr([upos(9,1)+0.15 upos(9,2)+0.05],[upos(6,1) upos(6,2)+0.05],'r',2,0.05)
drawarr([upos(4,1)+0.23 upos(4,2)-0.05],[upos(6,1)+0.23 upos(10,2)+0.15],'g',2,0.05)
drawarr([upos(8,1)+0.39 upos(8,2)-0.05],[upos(9,1)+0.39 upos(9,2)+0.15],'g',2,0.05)

% Alle Texte
% ==========
for k=1:length(txt)
   htx(k) = text(tpos(k,1),tpos(k,2),txt{k},'FontSize',fsz(k),'Color',col(k,:));
end

% Interaktivitt
% ==============
uip = axpos2figpos(upos);
uiq = uicontrol(fh,'Position',[uip(1,:) 0.05 0.05],'String','2','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(1)','BackgroundColor',[1 1 1]);
                 
uin = uicontrol(fh,'Position',[uip(2,:) 0.05 0.05],'String','4','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(2)','BackgroundColor',[1 1 1]);
                 
uia = uicontrol(fh,'Position',[uip(3,:) 0.15 0.05],'String','2','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(3)');
                 
uib = uicontrol(fh,'Position',[uip(4,:) 0.15 0.05],'String','3','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(4)');
                 
uio = uicontrol(fh,'Position',[uip(5,:) 0.055 0.05],'String','+','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(5)','ForegroundColor','g',...
                    'BackgroundColor',0.3*[1 1 1]);
                 
uir = uicontrol(fh,'Position',[uip(6,:) 0.15 0.05],'String','5','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(6)');
                 
uira = uicontrol(fh,'Position',[uip(7,:) 0.26 0.05],'String','0010','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(7)');
                 
uirb = uicontrol(fh,'Position',[uip(8,:) 0.26 0.05],'String','0011','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(8)');
                 
uir1 = uicontrol(fh,'Position',[uip(9,:) 0.26 0.05],'String','0101','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(9)');
                 
uir2 = uicontrol(fh,'Position',[uip(10,:) 0.15 0.05],'String','5','Style','edit',...
                    'Units','normalized','FontSize',fsiz,'FontName','FixedWidth',...
                    'CallBack','inforeprescbfcn(10)');
                 
% UserData des Fensters fr die Interaktivitt
% ============================================
fud.helptext = helptxt;
fud.handles  = {uiq;uin;uia;uib;uio;uir;uira;uirb;uir1;uir2};
fud.data     = {'2';'4';'2';'3';'+';'5';'010';'011';'+';'5';'101'};
set(fh,'UserData',fud,'KeyPressFcn','helpfcn');
