function [b,g,e,i] = isgru(t)
% [b,g,e,i] = isgru(t)
% Prfung, ob die quadratische Matrix t von Indices 
% in diese Tabelle als Operationstabelle die Axiome 
% einer endlichen Gruppe erfllt.
%
% b gibt an, ob t eine Gruppe definiert (1 sonst 0)
%
% Der boolesche Vektor g gibt an, welche Gruppenaxiome gelten:
% g(1) :  Assoziativgesetz
% g(2) :  Existenz des neutralen Elements
% g(3) :  Existenz der inversen Elemente
% g(4) :  Kommutativgesetz
%
% e    :  Index des neutralen Elements
%
% i    :  Indices der inversen Elemente
%
% isgru beweist durch vollstndige Prfung aller Kombinationen.
% Die Komplexitt ist proportional  n^3 .

%  Autor:  Klaus von der Heide,  v.d.heide@on-line.de

b = 0;
g = zeros(1,4);
e = [];
i = [];
[m,n] = size(t);
j = 1:n;

% Prfen, ob t quadratisch und sinnvolle Elemente enthlt
% =======================================================
if m~=n|any(t<1)|any(t>n)  break; end

% Finden des neutralen Elements
% =============================
a = 1;
for k=1:n
   a = a & all(t(t,k)' == t(k,t));
   ev(k) = all(t(k,:)==j & (t(:,k)==j')');
end
g(1) = a;

e = find(ev);
if length(e)==1  
   g(2) = 1;
   
   % Finden der inversen Elemente
   % ============================
   for k=1:n
      ii = find(t(k,:)==e & (t(:,k)==e)');
      if length(ii)>1 | length(ii)<1  
         i(k) = 0;
      else
         i(k) = ii;
      end
   end
   if all(i)
      g(3) = 1;
   end
end

% Prfung des Kommutativgesetzes
% ==============================
if all(t==t')  g(4) = 1; end

% Wenn Axiome 1,2,3 erfllt sind, liegt eine Gruppe vor
% =====================================================
b = all(g(1:3));

