function txt = morsesound2text(snd,fs,mxbpm,diag)
% txt = morsesound2text(snd,fs,mxbpm,diag)
% Dekodierung tnender Morsetelegraphie in Text.
% snd  :  Samplevektor
% fs   :  Abtastrate des Samplevektors in Hz
% mxbpm:  Grenze der Tastgeschwindigkeit in Buchstaben/Minute
%         Die tatschliche Tastgeschwindigkeit muss im Bereich 
%         1/2 mxbpm bis mxbpm liegen.
%         Ausserdem ist einzuhalten:  mxbpm < fs 
% diag :  falls dieser Parameter mit beliebigem Wert ~=0 
%         angegeben ist, gibt es eine Reihe von Graphiken
%
% Stand:  Das Programm arbeitet ordentlich. Allerdings:
% !!! Einige Parameter knnten noch optimiert werden:
% !!! esw, bw, bw1
% !!! Kontrollausgaben und auskommentierten Code noch wegnehmen
% !!! warum mssen die Funktionen eingauss, zweigauss, doppelgauss 
% !!! extern vorliegen ???
% !!! Wunsch:  Trgerregelung und kohrente Demodulation, wenn sinnvoll.
% !!! Modus "Aurora" einfhren mit breitem Signalfilter bw1

% Autor:  K. v. d. Heide,  e-mail:  v.d.heide@on-line.de

if nargin<1   snd = morsesound; end
if nargin<2    fs = 4000;       end
if nargin<3 mxbpm = 150;        end
if nargin<4  diag = 0;          end

% Konstanten
% ==========
wt   = 0.46;          % Wahrscheinlichkeit des getasteten Zustands
nfft = 1024;          % FFT-Lnge
nffe = 1024;          % FFT-Lnge fr Tastfrequenz
fsiz = 13;            % FontSize
g    = 30;            % Filtergrad fr Hilbertfilter
g2   = 30;            % Filtergrad des Energiefilters
%bw1  = 0.8*mxbpm     % Bandbreite des ersten Tiefpassfilters in Hz
%bw   = 0.12*mxbpm    % Bandbreite des Energiefilters in Hz
bw1  = 0.57*mxbpm;    % Bandbreite des ersten Tiefpassfilters in Hz
bw   = 0.085*mxbpm;   % Bandbreite des Energiefilters in Hz
sw   = 0.005;         % Schwelle fr die Tasterkennung
esw  = -0.20;         % Energieschwelle
xmxh = [0 0.06 0.07 0.09 0.13 0.14 0.16 100];  % Korrekturtabelle x
bpmf = [0.6 0.6 0.65 0.80 0.92 0.95 1   1];    % Korrekturtabelle y
bpm  = mxbpm*0.71;    % nullte Schtzung der Tastgeschwindigkeit

% Berechnung des Spektrums um den Ton zu finden
% =============================================
[ps,fps] = spectrum(snd,nfft,0,hanning(nfft),fs); % Spektrum
[mx,j] = max(ps(:,1));     % Maximum des Spektrums
f = fps(j(1));             % Tonfrequenz

%ttt = lsqcurvefit('eingauss',[700 50 1],fps,ps(:,1)); % funktioniert nicht

% Transformation ins komplexe Basisband
% =====================================
xa = snd.*exp(-i*2*pi*f/fs*(0:length(snd)-1));

% Signalfilter und Dezimation
% ===========================
if 2*bw1/fs>0.5
   xaf  = xa;
   srate1 = 1;
   fs1  = fs;
   dly1 = 0;
else
   [koef1,srate1,dly1,aw1] = mrdesign(bw1/fs,bw1/fs,1,70,0);
   fs1 = fs*prod(srate1);
   xaf = mrfilter(koef1,srate1,xa);
end

if 6*bw/fs1>1  error('BPM ist zu gross'); end

% Energie im Zeitbereich
% ======================
xaf = xaf/sqrt(sum(abs(xaf).^2)/length(xaf)); % Normierung auf mittlere Energie = 1
en = abs(xaf).^2;                       % Energie

% Energiefilter
% =============
koef2 = remez(g2,[0 2*bw/fs1 4*bw/fs1 1],[1 1 0 0]);
enf = filter(koef2,1,en);
enf = enf - mean(enf);
enq = enf.^2;
enq = enq/sqrt(sum(abs(enq).^2)/length(enq));


% Verteilung der gefilterten Energie
% ==================================
henf = hist(en,50);

% Lngenverteilung der Morsesymbole
% =================================
kk  = round(6*fs1/mxbpm);
eng = enf(kk:end);
ddd = [1  find(abs(diff(eng>esw))>0)  length(eng)]; % Indices fr Symbolnderungen
lsy = diff(ddd);                       % Lngen der Symbole
nhst= 100;
if eng(1)>0 anf = 1; else anf = 2; end
[hst,xhst] = hist(lsy(anf:2:end),nhst);
%ih0 = find(hst>0);
%hst = hst(ih0);
%xhst = xhst(ih0);
hst  = hst/sum(hst);
[mxh,imx] = max(hst);                  % je grsser, desto kleiner ist sigma
%mhx  = mean(hst.*xhst);                % Schwerpunkt des Histogramms ==> 1/bpm
mhx  = sum(hst.*xhst);                 % Schwerpunkt des Histogramms ==> 1/bpm
%bpm1 = 0.11*fs1/mhx*interp1(xmxh,bpmf,mxh);  % Konstante noch justieren und Abhngigkeit von mxh!
bpm1 = 10.75*fs1/mhx;
bpm2 = 6.2*fs1/xhst(imx);

mhx
[bpm,bpm1,bpm2]

if mxh>0.15  bpm = (2*bpm2 + bpm1)/3;
else         bpm = (bpm2 + bpm1)/2;
end

% Tastlngenentscheidung
% ======================
'doppelgauss:'
p1 = 6*fs1/bpm
ttt = lsqcurvefit('doppelgauss',[6*fs1/bpm  0.1],xhst,hst);

ttt(1:2)

bpm = (bpm + 6*fs1/ttt(1))/2;
bpmit(1) = bpm;

for iter=2:3                      % Beginn der Iterationsschleife
   
% Verbesserte Werte fr Iterationsschritt
% =======================================
%bw1 = 0.5*bpm        % Bandbreite des ersten Tiefpassfilters in Hz
bw1 = 0.3*bpm        % Bandbreite des ersten Tiefpassfilters in Hz
%bw  = 0.07*bpm       % Bandbreite des Energiefilters in Hz
%bw  = 0.06*bpm       % Bandbreite des Energiefilters in Hz
bw  = 0.05*bpm       % Bandbreite des Energiefilters in Hz

% Signalfilter und Dezimation
% ===========================
if 2*bw1/fs>0.5
   xaf  = xa;
   srate1 = 1;
   fs1  = fs;
   dly1 = 0;
else
   [koef1,srate1,dly1,aw1] = mrdesign(bw1/fs,bw1/fs,1,70,0);
   fs1 = fs*prod(srate1);
   xaf = mrfilter(koef1,srate1,xa);
end

% Energie im Zeitbereich
% ======================
xaf = xaf/sqrt(sum(abs(xaf).^2)/length(xaf)); % Normierung auf mittlere Energie = 1
en  = abs(xaf).^2;                       % Energie
ens = en - mean(en);
ens = ens/sqrt(ens*ens'/length(ens));    % Skaliert und +-0

% Energiefilter
% =============
koef2 = remez(g2,[0 2*bw/fs1 5*bw/fs1 1],[1 1 0 0]);
%koef2 = remez(g2,[0 2*bw/fs1 4*bw/fs1 1],[1 1 0 0]);
%koef2 = fir1(g2,0*bw/fs1);
enf = filter(koef2,1,ens);
enf = enf - mean(enf);
eqf = enf.^2;
eqf = eqf/sqrt(sum(abs(eqf).^2)/length(eqf));

% Lngenverteilung der Morsesymbole
% =================================
kk  = round(6*fs1/bpm);
eng = enf(kk:end);
ddd = [1  find(abs(diff(eng>esw))>0)  length(eng)]; % Indices fr Symbolnderungen
lsy = diff(ddd);                       % Lngen der Symbole
if eng(1)>0 anf = 1; else anf = 2; end
[hst,xhst] = hist(lsy(anf:2:end),nhst);
%ih0 = find(hst>0);
%hst = hst(ih0);
%xhst = xhst(ih0);
hst = hst/sum(hst);
[mxh,imx] = max(hst);                  % je grsser, desto kleiner ist sigma
%mhx = mean(hst.*xhst);                 % Schwerpunkt des Histogramms ==> 1/bpm
mhx = sum(hst.*xhst);                  % Schwerpunkt des Histogramms ==> 1/bpm
%bpm1 = 0.11*fs1/mhx*interp1(xmxh,bpmf,mxh);  % Konstante noch justieren und Abhngigkeit von mxh!
bpm1 = 10.75*fs1/mhx;
bpm2 = 6.2*fs1/xhst(imx);

'iter'
mhx
[bpm,bpm1,bpm2]

if mxh>0.15  bpm = (2*bpm2 + bpm1)/3;
else         bpm = (bpm2 + bpm1)/2;
end

bpm, p1=6*fs1/bpm

% Tastlngenentscheidung
% ======================
'doppelgauss:'
p1 = 6*fs1/bpm
ttt = lsqcurvefit('doppelgauss',[6*fs1/bpm  0.1],xhst,hst);

ttt(1:2)

bpm = 5.75*fs1/ttt(1);

bpm, 
'zweigauss:'
p1=5.75*fs1/bpm

rrr = lsqcurvefit('zweigauss',[6*fs1/bpm*[1  3]  3  mxh],xhst,hst);
rrr(2) = min(rrr(2),3.5*rrr(1));
bpmit(iter) = 24.2*fs1/(2*ttt(1) + rrr(1) + rrr(2)/3);

rrr,
bpmit(iter)

vvv = bpmit(iter)/bpmit(iter-1);
if (vvv < 1.2) & (1/vvv < 1.2)  break; end

end                                    % Ende der Iterationsschleife

bpmit

% Einfache Symbolerkennung anhand der Lnge in ens
% ================================================
mpl = (2*ttt(1) + rrr(1) + rrr(2)/3)/4; % mittlere Punktlnge
ppp = 2*mpl;                            % Entscheidungsschwelle fr Strichlnge
ss1 = 2*mpl;                            % Entscheidungsschwelle fr Zeichentrenner
ss2 = 3.4*mpl;                          % Entscheidungsschwelle fr Worttrenner
%ss2 = 3.7*mpl;                          % Entscheidungsschwelle fr Worttrenner
sy  = '';

[ppp,ss1,ss2]

for k=1:length(lsy)
   if (lsy(k)>ppp)&(eng(ddd(k)+1)>esw)
      sy = [sy  '-'];
   elseif eng(ddd(k)+1)>esw 
      sy = [sy  '.'];
   elseif lsy(k)>ss1
      sy = [sy  ' '];
      if lsy(k)>ss2  sy = [sy  ' ']; end
   end
end
txt = morsebits2text(morse2bits(sy))          % Ausgabe des dekodierten Textes

if ~diag  break; end

% Graphische Darstellungen
% ========================
fh11 = fullscreen('Signalspektrum','w',0,[3 3; 1 1]);
plot(fps,ps(:,1))
title('Signalspektrum','FontSize',fsiz)
xlabel('Tonfrequenz [Hz]','FontSize',fsiz)

fh12 = fullscreen('Verteilung','w',0,[3 3; 1 2]);
hist(enf,50)
title('Energieverteilung','FontSize',fsiz)
xlabel('normierte Energie','FontSize',fsiz)

fh13 = fullscreen('Tastverteilung','w',0,[3 3; 1 3]);
bar(xhst,hst)
xxx = (0:200)/200*xhst(end);
line(xxx,eingauss([rrr(1) rrr(3) rrr(4)],xxx),'Color','r')
line(xxx,eingauss([rrr(2) rrr(3) 0.6*rrr(4)],xxx),'Color','g')
set(gca,'XLim',[0 xhst(end)])
title('Lngenverteilung der Morsesymbole','FontSize',fsiz)
xlabel('Anzahl der Abtastungen','FontSize',fsiz)
legend('Punkte','Striche')

if 2*bw1/fs<=0.5
   fh21 = fullscreen('Filter','w',0,[3 2; 2 1]);
   filterplot2(koef1,0,fs/2,'log',-100,+10,cumprod([1 1./srate1(1:end-1)]),24,1,fs)
   title('Multiratenfilter','FontSize',fsiz)
end

fh22 = fullscreen('Filter','w',0,[3 2; 2 2]);
filterplot2(koef2,0,fs1/2,'log',-40,+10,1,24,0,fs1)
title('Basisbandfilter','FontSize',fsiz)

fh3 = fullscreen('Energie gefiltert','w',3,[3 1; 3 1]);
axes('Position',[0 0.1 1 0.8])
enf = eng;
plot(enf)
line([1 length(enf)],[0 0],'color',0.7*[1 1 1])
for k=1:length(ddd)-1
   if eng(ddd(k)+1)>esw  line(0.5+ddd(k:k+1),[1 1]*esw,'Color','r','LineWidth',1.5); end
end
axis([1 length(enf) min(enf)-0.1  1.1*max(enf)]) 
axis off
title('Energie gefiltert','FontSize',fsiz)
xlabel('Zeit','FontSize',fsiz)

%===============================================================================

function  y = eingauss(p,x)
ss2 = 2*p(2)^2;
y = p(3)*exp(-((x-p(1)).^2)/ss2);

%===============================================================================

function  y = zweigauss(p,x)
a1  = p(4);         % !!!! noch einsetzen als Funktion von p(3)
a2  = 0.7*a1;       % !!!! Faktor noch przisieren
ss2 = 2*p(3)^2;
y = a1*exp(-((x-p(1)).^2)/ss2) + a2*exp(-((x-p(2)).^2)/ss2);

%===============================================================================

function  y = doppelgauss(p,x)
ss2 = 2*(0.3*p(1)).^2;
y = p(2)*(1.5*exp(-(x-p(1)).^2/ss2) + exp(-(x-3*p(1)).^2/ss2));