function txt = morsesound3text(snd,fs,diag)
% txt = morsesound3txt(snd,fs,diag)
% Dekodierung tnender Morsetelegraphie
% snd  :  Samplevektor
% fs   :  Abtastrate des Samplevektors in Hz
% diag :  falls dieser Parameter mit beliebigem Wert angegeben ist,
%         gibt es eine Reihe von Graphiken

if nargin<1  snd = morsesound; end
if nargin<2   fs = 4000;       end

% Konstanten
% ==========
bpm  = 120;           % Grssenordnung der Buchstaben pro Minute
nfft = 128;           % FFT-Lnge
nffe = 4096;          % FFT-Lnge fr Spektrum des Amplitudenverlaufs
fsiz = 13;            % FontSize

% Berechnung des Spektrums um den Ton zu finden
% =============================================
[ps,fps] = spectrum(snd,nfft,0,hanning(nfft),fs); % Spektrum
[mx,j] = max(ps(1,:));     % Maximum des Spektrums
f  = fps(j(1));       % Tonfrequenz relativ zur Abtastfrequenz

% Transformation ins komplexe Basisband
% =====================================
x  = snd.*exp(-i*2*pi*f*(0:length(snd)-1));

% Erstes Tiefpassfilter
% =====================
[b1,a1] = butter(2,bpm/2/fs);
y = filter(b1,a1,x);

% Berechnung des Spektrums der Energie um die Tastrate zu finden
% ==============================================================
[pe,fpe] = spectrum(y.*conj(y),nffe,0,hanning(nffe),fs);
j   = find(fpe>2&fpe<10);
pee = pe(j);
fee = fpe(j);
[my,j] = max(pee);    % Maximum des Spektrums
ft = fee(j(1));       % Tastfrequenz relativ zur Abtastfrequenz

% Angepasstes Tiefpassfilter
% ==========================
[b2,a2] = butter(2,50*ft/fs);
z = filter(b2,a2,y);

% Bestimmung der Schwelle fr die Datenentscheidung
% =================================================
%zz = filter(b2,a2,z.*conj(z)); % Energie
zz = z.*conj(z);
sw = mean(zz)/2;      % primitive Schwelle

% Datenentscheidung
% =================
d  = zz>sw;

% Dekodierung in Bits
% ===================


% Umsetzung der Bits in Text
% ==========================

if ~diag  break; end

% Graphische Darstellungen
% ========================
fh1 = fullscreen('Signalspektrum','w',2,[3 3; 1 1]);
plot(fps,ps(:,1))
title('Signalspektrum','FontSize',fsiz)
xlabel('Tonfrequenz [Hz]','FontSize',fsiz)

fh2 = fullscreen('Tastspektrum','w',2,[3 3; 2 1]);
plot(fpe,pe(:,1))
set(gca,'XLim',[0 30])
title('Spektrum des Amplitudenverlaufs','FontSize',fsiz)
xlabel('Frequenz [Hz]','FontSize',fsiz)

fh3 = fullscreen('Verteilung','w',2,[3 3; 1 2]);
hist(zz)

fh4 = fullscreen('Signalbetrag','w',2,[3 3; 2 2]);
plot(sqrt(zz))

fh5 = fullscreen('Filter 1','w',2,[3 3; 1 3]);
filterplot({{b1;a1}},0,0.1,'log',-60,+10)

fh6 = fullscreen('Gesamtfilter','w',2,[3 3; 2 3]);
filterplot({{b1;a1};{b2;a2}},0,0.1,'log',-60,+10,[1 1],24,1)

fh7 = fullscreen('Datenentscheidung','w',2,[6 1; 5 1]);
plot(d)
set(gca,'YLim',[0 1.5])

ende=0;