function ca = obdd(binexpr,variable)
% ca = function obdd(binexpr,variable)
% Umsetzung des binralgebraischen Ausdrucks binexpr 
% mit den Variablen variable in ein CellArray fr das OBDD.
% Anstelle des Ausdrucks kann auch ein binrer Vektor der 
% Lnge 2^n mit der Funktionstabelle angegeben werden.
% Wird die Ausgabe nicht zugewiesen, so gibt es eine Graphik.
%
% binexpr  :  String fr den schaltalgebraischen Ausdruck
%             Beispiel:  '(a1|b) & (~a2|c) & b|c'
%
% variable :  Cell array mit den Variablen als Strings
%             Folgende Variablennamen sind nicht erlaubt:
%             jjjjj,kkkkk,ftab,binexpr,eval,sprintf,variable
%             sowie alle Schlsselwrter von MATLAB wie for, end, ...
%             Beispiel:  {'a1'; 'a2'; 'b'; 'c'}

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<2  variable = {'a';'b';'c';'d'};   end
if nargin<1   binexpr = '(a&c) | (~b&d&~c)'; end

% Konstanten
% ==========
r = 0.5;                              % Radius fr Knoten
col = 0.8*[1 1 1];                    % Fllfarbe fr Knoten
lc  = 'rg';                           % Farben der Kanten fr 0 bzw. 1
lw  = 1.5;                            % Dicke der Kanten

% Berechnung der Funktionstabelle
% ===============================
if ischar(binexpr)
   jjjjj = '';
   for kkkkk=1:length(variable)
      jjjjj = [jjjjj sprintf('%sfor %s=0:1\n',blanks(3*kkkkk),variable{kkkkk})];
   end
   jjjjj = [jjjjj sprintf('%sftab = [ftab eval(binexpr)];\n',blanks(3*(length(variable)+1)))];
   for kkkkk=length(variable):-1:1
      jjjjj = [jjjjj sprintf('%send\n',blanks(3*kkkkk))];
   end
   ftab = '';
   eval(jjjjj);
else
   ftab = binexpr;
end
n = round(log2(length(ftab)));

% Streichung von Knoten
% =====================
s = ones(1,2^n);
for k=1:n
   a = 2^(n-k)
   for j=1:2^(k-1)
      if all(ftab((2*j-2)*a+(1:a))==ftab((2*j-1)*a+(1:a)))
         s((2*j-1)*a+(1:a)) = 0;
      end
   end
end
[ftab;s]

% Graphik
% =======
dy = (2^n + 3)/(n+1)/1.333;           % vertikaler Abstand der Ebenen 
fsiz = round(400*2^-n);
fh = fullscreen('OBDD');              % Fenster
axes('Position',[0 0 1 1])            % Achsen
for k=1:n                             % Ebene
   for j=1:2^(k-1)                    % Knoten
      if s(1+2^(n-k)+(j-1)*2^(n-k+1)) % Knoten vorhanden
         drawccir([2^(n-k)+(j-1)*2^(n-k+1) (n-k+1.5)*dy],r,col); 
      end
   end
end
for k=1:2^n
   if s(k)
      z = 1;
      while k+z<=2^n & s(k+z)==0  
         z = z + 1; 
      end
      dk = log2(z);
      drawcsq([k+z/2-1  1],2*r,col,num2str(ftab(k)),fsiz);
      z
      z/2*(1-2*(rem(k,z)>z/2))
      line(k+z/2-1+[0 z/2*(1-2*(1+rem((k-1),2*z)>z))],[1+r  (1.5+dk)*dy-r],'LineWidth',lw,'Color',lc(1+ftab(k)))
   end
end
axis([-2 2^n+2 0 (n+1)*dy]);  axis off
break
for k=1:n                             % Ebene
   for j=1:2^(k-1)                    % Knoten
      if s(1+2^(n-k)+(j-1)*2^(n-k+1)) % Knoten vorhanden
         for i=1:2
            if s(1+2^(n-k-1)+(i-1)*2^(n-k)) % Knoten vorhanden
            %         line([2^(n-k)+(j-1)*2^(n-k+1) (n-k+1.5)*dy],r,col);
            end
         end
      end
   end
end


