function  z = strmult(x,y,q)
% z = strmult(x,y)
% Multiplizierer fr ganze Zahlen zur Basis q, die als Strings vorliegen.
% Negative Zahlen werden durch ein voranstehendes "-" gekennzeichnet.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Konstante Ziffern
% =================
charzif = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

% Prfung der Parameter
% =====================
lz = 1 + max(length(x),length(y));
if (q<2)|(q>length(charzif)) error('strplus: Basis nicht aus 2:64'); end
if (x(1)=='-')|(x(1)=='+')  xx = x(2:end); else xx = x; end
if (y(1)=='-')|(y(1)=='+')  yy = y(2:end); else yy = y; end
if checkchar(xx,charzif(1:q)) error('strplus: x mit falscher Ziffer'); end
if checkchar(yy,charzif(1:q)) error('strplus: y mit falscher Ziffer'); end
lx = length(xx);
ly = length(yy);

% Multiplikationstabelle
% ======================
for i=1:q
   for j=1:q
      mt(i,j) = (i-1)*(j-1);
   end
end

% Multiplikation  
% ==============
c = 0;
for k=ly+lx:-1:1                    % alle Stellen des Resultats 
   zk = c;                          % bertrag als Anfangswert der Summation
   for i=lx:-1:1                    % alle Stellen des ersten Operanden
      if (k-i>0) & (k-i)<=ly        % nur wenn die Stelle existiert
         zk = zk + mt(find(xx(i)==charzif),find(yy(k-i)==charzif));
      end
   end
   z(k) = num2basstr(rem(zk,q),q);  % k-te Ziffer des Resultats
   c = floor(zk/q);                 % bertrag in die (k+1)-te Stelle
end

% Fhrende Nullen streichen
% =========================
while (z(1)=='0') & (length(z)>1)  z = z(2:end); end

% Vorzeichen
% ==========
if rem((x(1)=='-')+(y(1)=='-'),2)==1   z = ['-' z]; end


