%
%
%10.4  Flipflop-Typen %<'FontWeight',MSVTITLWEIGHT,'FontSize',MSVTITLSIZE,'Color',MSVTITLCOLOR>
%
%
%
% Flipflops  sind nicht-getaktete einfache Schaltwerke, die als wiederbeschreibbarer 
% Speicher fr ein Bit eingesetzt werden.  Man unterscheidet sie in zwei Aspekten:
%
% (1)  Nach der Art, wie das Bit gesetzt bzw. gelscht wird:
%       R-S-Flipflop,  J-K-Flipflop,  D-Flipflop
%
% (2)  Ob (und wie) ein Eingang als "Takt" verwendet wird:
%       R-S-Latch,  Pegel-Steuerung,  Flanken-Steuerung
%
% Man beachte, dass Flipflops immer nicht-getaktete Schaltwerke sind, weil die 
% Rckkopplung nur Verzgerungen enthlt.  Der "Takt" ist aus Sicht des Flipflops 
% ein Eingang des Schaltwerkes, und zwar jener, der bei Einsatz des Flipflops 
% als Takt-gesteuertes Zeitglied in einem getakteten Schaltwerk der Takt dieses 
% getakteten Schaltwerks ist.
%
% Nachfolgend sind fr die wichtigsten Typen jeweils das Zustandsdiagramm und die 
% Flusstafel sowie ein typisches Signalspiel und das Schaltbild einer typischen 
% Realisierung angegeben.
%
%
% \bfPegel-gesteuertes R-S-Flipflop
%
% Der einfachste Flipfloptyp ist das R-S-Flipflop (auch S-R-Flipflop oder NOR-Basis-Flipflop).
% Das folgende Bild zeigt das Moore-Modell inklusive der logischen Verknpfungen:
% 
drawnorffmoore   % Moore-Modell
%
% Dem Modell entnimmt man die schaltalgebraischen Ausdrcke fr die beiden Zustandsbits:
%
% z_{1next} = Q  = NOR(z_2, R)) = ~(z2|R) %<'FontName',MSVFORMFONT>
% z_{2next} = Q' = NOR(z_1, S)) = ~(z1|S) %<'FontName',MSVFORMFONT>
%
% Hieraus ergibt sich die Flusstafel:
%
% Zustand Zustand-| Input-Index:      1       2       3       4    %<'FontName',MSVFORMFONT> 
%  Z1,Z2   Index  | Bitmuster S,R:   00      01      11      10    %<'FontName',MSVFORMFONT>
% ---------------------------------------------------------------- %<'FontName',MSVFORMFONT>
%   00       1    | Folgezustand    11(3)   01(2)   00(1)   10(4)  %<'FontName',MSVFORMFONT>
%   01       2    | Folgezustand    01(2)   01(2)   00(1)   00(1)  %<'FontName',MSVFORMFONT>
%   11       3    | Folgezustand    00(1)   00(1)   00(1)   00(1)  %<'FontName',MSVFORMFONT>
%   10       4    | Folgezustand    10(4)   00(1)   00(1)   10(4)  %<'FontName',MSVFORMFONT>
%
% Die Zustands-Indices tragen wir in die Matrix  {\fontname{Courier}utab}  ein und geben in  {\fontname{Courier}zcod}  und  {\fontname{Courier}xcod} 
% die Binrcodierung fr die jeweiligen Indices an.  In  {\fontname{Courier}xnam}  notieren wir noch die blichen 
% Bezeichnungen fr die Eingnge in der Reihenfolge, die wir fr die Bits der Codierung 
% vorgesehen haben:
%
clear utab
utab = [3 2 1 4; 2 2 1 1; 1 1 1 1; 4 1 1 4]
zcod = {'00';'01';'11';'10'};
xcod = {'00';'01';'11';'10'};
xnam = 'S,R';
demofsm(utab,zcod,2,1,xcod,xnam)
%
% Ein typisches Signalspiel zeigt:
%
S = [0 0 0 1 0 0 0 1 0 1 0 1 0 0 0 1 1 0 0 0 0 0]; % Setzeingang
R = [0 1 0 0 0 1 0 0 0 0 0 0 0 1 0 1 1 0 0 0 1 0]; % Rcksetzeingang
xnam = {'S';'R'};
insigs = nfach([S; R],10);          % Inputmatrix fr drawswsigs
delta = {'~(R|z2)';'~(S|z1)'};      % berfhrungsfunktion
dly = {ceil(3*rand); ceil(3*rand)}; % zufllige Verzgerungen aller Verknpfungen
drawswsigs(delta,xnam,dly,insigs);  % Simulation
%% Rufen Sie dies mehrfach auf, da der Anfangszustand und die Verzgerungen zufllig 
%% gewhlt werden.
%
% Das Schaltbild des NOR-Basis-Flipflops ist:
%
drawnorff        % Schaltbild
%
%
% \bfTaktpegel-gesteuertes R-S-Flipflop
%
% Aus dem Schaltbild
%
drawprsff    % Schaltbild
%
% entnimmt man die schaltalgebraischen Ausdrcke fr die beiden Zustandsbits:
%
% z_{1next} = Q  = NOR(z_2, AND(R, C)) = ~(z2|(R&C)) %<'FontName',MSVFORMFONT>
% z_{2next} = Q' = NOR(z_1, AND(S, C)) = ~(z1|(S&C)) %<'FontName',MSVFORMFONT>
%
% Hieraus ergibt sich die Flusstafel:
%
% Zustand Zustand-| Input-Index:     1     2     3     4     5     6     7     8   %<'FontName',MSVFORMFONT> 
%  Z1,Z2   Index  | Bitmuster S,R,C:000   001   010   011   100   101   110   111  %<'FontName',MSVFORMFONT>
% -------------------------------------------------------------------------------- %<'FontName',MSVFORMFONT>
%   00       1    | Folgezustand   11(3) 11(3) 11(3) 01(2) 11(3) 10(4) 11(3) 00(1) %<'FontName',MSVFORMFONT>
%   01       2    | Folgezustand   01(2) 01(2) 01(2) 01(2) 01(2) 00(1) 01(2) 00(1) %<'FontName',MSVFORMFONT>
%   11       3    | Folgezustand   00(1) 00(1) 00(1) 00(1) 00(1) 00(1) 00(1) 00(1) %<'FontName',MSVFORMFONT>
%   10       4    | Folgezustand   10(4) 10(4) 10(4) 00(1) 10(4) 10(4) 10(4) 00(1) %<'FontName',MSVFORMFONT>
%
% Die Zustands-Indices tragen wir in die Matrix  {\fontname{Courier}utab}  ein und geben in  {\fontname{Courier}zcod}  und  {\fontname{Courier}xcod} 
% die Binrcodierung fr die jeweiligen Indices an.  In  {\fontname{Courier}xnam}  notieren wir noch die blichen 
% Bezeichnungen fr die Eingnge in der Reihenfolge, die wir fr die Bits der Codierung 
% vorgesehen haben:
%
clear utab
utab(1,:) = [3 3 3 2 3 4 3 1];
utab(2,:) = [2 2 2 2 2 1 2 1];
utab(3,:) = [1 1 1 1 1 1 1 1];
utab(4,:) = [4 4 4 1 4 4 4 1]
zcod = {'00';'01';'11';'10'};
xcod = {'000';'001';'010';'011';'100';'101';'110';'111'};
xnam = 'S,R,C';
demofsm(utab,zcod,2,1,xcod,xnam)
%
% Ein typisches Signalspiel zeigt:
%
S = [0 1 1 1 0 0 0 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 0 0 0]; % Setzeingang
R = [0 0 0 0 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 1 1 1 1 1 0]; % Rcksetzeingang
C = [0 0 1 0 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 1 0 0 1 1 1 1 1 1 0 0]; % Takteingang
xname = {'S';'R';'C'};
insigs = nfach([S; R; C],10);          % Inputmatrix fr drawswsigs
delta = {'~(z2|(R&C))';'~(z1|(S&C))'}; % berfhrungsfunktion
dly = {[ceil(3*rand) ceil(3*rand)]; [ceil(3*rand) ceil(3*rand)]}; % zufllige Verzgerungen
drawswsigs(delta,xname,dly,insigs);     % Simulation
%% Rufen Sie dies mehrfach auf, da der Anfangszustand und die Verzgerungen zufllig gewhlt werden.
%
%
% \bfTaktpegel-gesteuertes D-Flipflop
%
% Wie bei den beiden vorangegangenen Flipfloptypen gehen wir aus von einem Schaltbild. 
% Wir nehmen dabei ein anderes als das im vorigen Abschnitt hergeleitete Earle-Latch:
% 
drawtpdff
%
% Dem Schaltbild entnimmt man die schaltalgebraischen Ausdrcke fr die beiden im Bild 
% berkreuz laufenden Zustandsbits:
%
% z_{1next} = Q  = NAND(z_2, NAND(C,D))          = ~(z2&~(C&D))      %<'FontName',MSVFORMFONT>
% z_{2next} = Q' = NAND(z_1, NAND(C, NAND(C,D))) = ~(z1&~(C&~(C&D))) %<'FontName',MSVFORMFONT>
%
% Hieraus ergibt sich die Flusstafel:
%
% Zustand Zustand-| Input-Index:      1       2       3       4    %<'FontName',MSVFORMFONT> 
%  Z1,Z2   Index  | Bitmuster D,C:   00      01      11      10    %<'FontName',MSVFORMFONT>
% ---------------------------------------------------------------- %<'FontName',MSVFORMFONT>
%   00       1    | Folgezustand    11(3)   11(3)   11(3)   11(3)  %<'FontName',MSVFORMFONT>
%   01       2    | Folgezustand    01(2)   01(2)   11(3)   01(2)  %<'FontName',MSVFORMFONT>
%   11       3    | Folgezustand    00(1)   01(2)   10(4)   00(1)  %<'FontName',MSVFORMFONT>
%   10       4    | Folgezustand    10(4)   11(3)   10(4)   10(4)  %<'FontName',MSVFORMFONT>
%
% Die Zustands-Indices tragen wir in die Matrix  {\fontname{Courier}utab}  ein und geben in  {\fontname{Courier}zcod}  und  {\fontname{Courier}xcod} 
% die Binrcodierung fr die jeweiligen Indices an.  In  {\fontname{Courier}xnam}  notieren wir noch die blichen 
% Bezeichnungen fr die Eingnge in der Reihenfolge, die wir fr die Bits der Codierung 
% vorgesehen haben:
%
clear utab
utab = [3 3 3 3; 2 2 3 2; 1 2 4 1; 4 3 4 4]
zcod = {'00';'01';'11';'10'};
xcod = {'00';'01';'11';'10'};
xnam = 'D,C';
demofsm(utab,zcod,2,1,xcod,xnam)
%
% Wir testen das Flipflop mit dem selben Input wie beim Earle-Latch:
%
D = [0 0 0 0 1 1 1 1 1 0 0];       % Dateninput
C = [0 1 0 1 1 0 1 0 1 1 0];       % Taktinput
insigs = nfach([D;C],50);          % Inputmatrix fr drawswsigs
dearle = {'(~C&z1)|(C&D)|(D&z1)'}; % berfhrungsfunktion Earle-Latch
dlye = {ceil(4*rand(1,4))};        % zufllige Verzgerungen aller Verknpfungen
drawswsigs(dearle,{'D';'C'},dlye,insigs);
dlatch = {'~(z2&~(C&D))';'~(z1&~(C&~(C&D)))'}; % berfhrungsfunktion D-Latch
dlyl = {ceil(4*rand(1,2)); ceil(4*rand(1,3))}; % zufllige Verzgerungen aller Verknpfungen
drawswsigs(dlatch,{'D';'C'},dlyl,insigs);
%
% Diese Form des D-Latches wird hufig verwendet.  Man muss jedoch beachten, dass der  Q'-Ausgang 
% einen statischen 1-Hasard aufweist.
%
%% bungsaufgabe:  Wodurch wird der Hasard des Q'-Ausgangs verursacht und kann man ihn beseitigen?
%
%
%
% \bfRckflanken-gesteuertes D-Flipflop
%
% Wie bei den beiden vorangegangenen Flipfloptypen gehen wir aus von einem Schaltbild. 
% Wir nehmen dabei ein anderes als das im Abschnitt 10.3 hergeleitete:
% 
drawetdff('-',1)
%
% Dem Schaltbild entnimmt man die schaltalgebraischen Ausdrcke fr die vier im Bild 
% angegebenen Zustandsbits:
%
% z_{1next} = NOR(z2, C, NOR(z1, D)       %<'FontName',MSVFORMFONT>
% z_{2next} = NOR(C, NOR(z2, NOR(z1, D))) %<'FontName',MSVFORMFONT>
% z_3 = NOR(z2, z4)                 %<'FontName',MSVFORMFONT>
% z_4 = NOR(z1, z3)                 %<'FontName',MSVFORMFONT>
%
% Hieraus ergibt sich die Flusstafel, die in diesem Fall mithilfe der Funktion 
% {\fontname{Courier}binexp2flusstafel}  automatisch erstellt wird:
%
delta = {'~(z2|C|~(z1|D))';'~(C|~(z2|~(z1|D)))';'~(z2|z4)';'~(z1|z3)'};
incod = {'D=0; C=0;'; 'D=0; C=1;'; 'D=1; C=1;'; 'D=1; C=0;'};
utab  = binexp2flusstafel(delta,incod)
%
% Damit ergibt sich das Zustandsdiagramm und die Graphik der Flusstafel:
%
zcod = {'0000';'0001';'0010';'0011';'0100';'0101';'0110';'0111'};
zcod = cat(1,zcod,{'1000';'1001';'1010';'1011';'1100';'1101';'1110';'1111'});
xcod = {'00';'01';'11';'10'};
xnam = 'D,C';
demofsm(utab,zcod,2,1,xcod,xnam)
%
%% Hier sollte man einige typische Signalfolgen durchspielen.  Der Ausgang Q ist z_3, 
%% also das dritte Bit der Zustandscodierung.
%
%
% Wir testen das Flipflop ebenfalls mit vorgegebenen Inputsignalen:
%
D = [0 0 0 0 1 0 1 1 0 0 0 0 0];   % Dateninput
C = [0 1 0 1 1 1 1 0 0 0 1 0 0];   % Taktinput
insigs = nfach([D;C],20);          % Inputmatrix fr drawswsigs
dly = {ceil(3*rand(1,2)); ceil(3*rand(1,3));ceil(3*rand);ceil(3*rand);};        % zufllige Verzgerungen aller Verknpfungen
delta = {'~(z2|C|~(z1|D))';'~(C|~(z2|~(z1|D)))';'~(z2|z4)';'~(z1|z3)'};
drawswsigs(delta,{'D';'C'},dly,insigs);
%
%
%
% \bfVorderflanken-gesteuertes D-Flipflop
%
% Durch Vergleich des folgenden Schaltbildes mit dem des rckflankengesteuerten Flipflops
% erkennt man, dass diese Typen gleich sind bis auf Inversion aller Variablen.  Daher 
% sind im Schaltbild NAND- und NOR-Verknpfungsglieder gegeneinander getauscht: 
%
drawetdff('+')
%
%
%
% \bfZweiflankengesteuertes-Flipflop
%
% Auch fr diesen Typ geben wir eine bliche Realisierungsform als Schaltbild an:
%
draw2etdff
%
% Es sei dem Leser als bungsaufgabe berlassen, die Rckfhrungen als Zustandsvariable
% zu definieren und die schaltalgebraischen Ausdrcke fr die berfhrungsfunktion zu 
% bestimmen.  Wegen der grossen Zahl theoretisch mglicher Zustnde (64) ist das 
% automatisch generierte Zustandsdiagramm schon sehr unbersichtlich.  Das Zustands-
% diagramm lsst sich aber nach dem im Abschnitt 10.3 beschriebenen Verfahren reduzieren 
% auf eines mit nur 6 Zustnden.  Es ist nachfolgend dargestellt:
% 
demofsm([2 1 1 2; 2 1 3 2; 5 3 3 5; 5 4 4 5; 5 6 4 5; 2 6 6 2],{'D';'A';'F';'C';'B';'E'},1,1,{'00';'01';'11';'10'})
%
%
% \bfJ-K-Flipflop
%
% J-K-Flipflops sind Taktflanken-gesteuerte universelle Flipflops, die man durch 
% die beiden Eingnge  J  und  K  wie folgt benutzen kann:
%
%    J  K   Funktion                                        %<'FontName',MSVFORMFONT>
%  -------------------------------------------------        %<'FontName',MSVFORMFONT>
%    0  0   keine Reaktion auf das Taktsignal               %<'FontName',MSVFORMFONT>
%    0  1   Rcksetzen                                      %<'FontName',MSVFORMFONT>
%    1  0   Setzen (wie  S=1, R=0 beim S-R-Flipflop)        %<'FontName',MSVFORMFONT>
%    1  1   Wechsel  0 \rightarrow 1  bzw.  1 \rightarrow 0 %<'FontName',MSVFORMFONT>
%
% Hufig findet man auch J-K'-Flipflops, bei denen also das K-Signal invertiert 
% eingegeben wird. 
%
% J-K-Flipflops kann man auffassen als sehr einfache getaktete Schaltwerke mit einem
% gewhnlichen D-Flipflop als Zeitglied.  Das folgende Bild zeigt das Modell:
%
drawjkff
% 
% Die Bedeutung der J-K-Flipflops als Standard-Bausteine hat mit der Integration 
% grsserer Systeme auf Mikrochips abgenommen, weil man dort beim Entwurf 
% jeweils die geeigneten Flipflops einsetzt und dadurch Chipflche einsparen kann.     
%