function [s,po] = tree2binexpr(tree,t,pi)
% s = tree2binexpr(tree)
% Wandlung eines Baumes in einen schaltalgebraischen Ausdrucks.
%
% tree :    geschachtelter CellArray mit den Teilausdrcken der Form
%           {Operationssymbol, Operand1, Operand2, ... }
%           Folgende Operationssymbole in expr sind zulssig: 
%           '&' und '*' (AND), '|' (OR), '+' (XOR), '~' (NOT),
%           '/&' und '/*' (NAND), '/|' (NOR), '/+' (XNOR)
%
% s    :    String des schaltalgebraischen Ausdrucks in MATLAB-Syntax.
%           AND wird als all, NAND als ~all, OR als any und NOR als ~any
%           in Prfix-Notation geschrieben.
%
% Beispiel: tree2binexpr(binexpr2tree('~(~a|b)&c'))
%
% Siehe auch  binexpr2tree

% Die Variablen t und p werden fr die Rekursion bentigt.

% Autor:  Klaus von der Heide  e-mail:  v.d.heide@on-line.de

if nargin<1  help tree2binexpr; break; end
if nargin<2  t = '';  pi = 0; end

if ~iscell(tree)
   t  = [t(1:pi) ' ' tree  t(pi+1:end)];      % Variable einfgen
   pi = pi + length(tree) + 1;                % Index entsprechend schieben
else
   switch tree{1}                             % Verknpfung
      case {'&','*'},    op = 'all';   p = 5;
      case {'/&','/*'},  op = '~all';  p = 6;
      case '|',          op = 'any';   p = 5;
      case '/|',         op = '~any';  p = 6;
      case '+',          op = 'gfa';   p = 5;
      case '/+',         op = '~gfa';  p = 6;
   end
   if isempty(t)                              % Aufrufwurzel
      t  = [' '  op '([])'];
   else
      t  = [t(1:pi)  ' '  op  '([])'  t(pi+1:end)]; % Operation einfgen
   end
   pi = pi + p + 1;                           % Index auf Anfang Operanden
   for k=2:length(tree)
      [t,pi] = tree2binexpr(tree{k},t,pi);    % Operanden rekursiv
   end
   pi = pi + 2;                               % Index nach Klammer-zu
end
s  = t;
po = pi;